/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.schedule;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.google.common.util.concurrent.RateLimiter;
import com.zayac.admin.agent.model.entity.FinanceDO;
import com.zayac.admin.agent.model.entity.StatsDO;
import com.zayac.admin.agent.model.req.FinanceSumReq;
import com.zayac.admin.agent.service.FinanceService;
import com.zayac.admin.agent.service.FinanceSumService;
import com.zayac.admin.agent.service.StatsService;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.config.RateLimiterConfig;
import com.zayac.admin.constant.ApiPathConstants;
import com.zayac.admin.req.AgentDataVisualListReq;
import com.zayac.admin.req.PayRecordListReq;
import com.zayac.admin.req.PayRecordsListReq;
import com.zayac.admin.req.team.TeamFinanceReq;
import com.zayac.admin.req.team.TeamInfoReq;
import com.zayac.admin.req.team.TeamMemberReq;
import com.zayac.admin.resp.*;
import com.zayac.admin.resp.team.*;
import com.zayac.admin.service.*;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.model.resp.DeptUsersResp;
import com.zayac.admin.system.model.resp.UserWithRolesAndAccountsResp;
import com.zayac.admin.system.service.*;
import com.zayac.admin.utils.TableFormatter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Profile;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import top.continew.starter.core.exception.BusinessException;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.Executor;
import java.util.concurrent.atomic.AtomicIntegerArray;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

@Component
@RequiredArgsConstructor
@Slf4j
@Profile("prod")
public class DailyReport {
    private final TeamService teamService;
    private final DeptService deptService;
    private final TelegramMessageService telegramMessageService;
    private final AgentDataVisualListService agentDataVisualListService;
    private final StatsService statsService;
    private final CompletableFutureFinanceService completableFutureFinanceService;
    private final FinanceService financeService;
    private final FinanceSumService financeSumService;
    private final CompletableFutureWebClientService completableFutureWebClientService;
    private final Executor asyncTaskExecutor;
    private final RateLimiterConfig rateLimiterConfig;

    private static final String MINISTER_ROLE_CODE = "minister";
    private static final String SEO_TEAM_LEADER_ROLE_CODE = "seo_team_leader";
    private static final String ASSISTANT_ROLE_CODE = "assistant";
    private static final String SEO_ROLE_CODE = "seo";
    private static final String PLATFORM_HTH = "华体会";

    @Scheduled(cron = "0 40 11,14,17,21,23 * * ?")
    public void teamAccountDailyReport() {
        LocalDateTime nowDateTime = LocalDateTime.now();
        log.info("dailySummarize started at {}", nowDateTime);
        LocalDate nowDate = LocalDate.now();
        //查询部门下的所有用户
        List<DeptUsersResp> deptWithUsersAndAccounts = deptService.getDeptWithUsersAndAccounts(MINISTER_ROLE_CODE);
        deptWithUsersAndAccounts.forEach(dept -> {
            //根据用户角色对部门用户进行分组
            Map<String, List<UserWithRolesAndAccountsResp>> usersByRole = dept.getUsers()
                .stream()
                .flatMap(user -> user.getRoles()
                    .stream()
                    .map(role -> new AbstractMap.SimpleEntry<>(role.getCode(), user)))
                .collect(Collectors.groupingByConcurrent(Map.Entry::getKey, Collectors
                    .mapping(Map.Entry::getValue, Collectors.toList())));

            var ministerUser = usersByRole.get(MINISTER_ROLE_CODE).get(0);
            //获取账号不为空的用户
            var deptUsers = dept.getUsers().stream().filter(user -> CollUtil.isNotEmpty(user.getAccounts())).toList();
            var assistants = usersByRole.get(ASSISTANT_ROLE_CODE);
            sendDailyReport(nowDate, nowDate.atStartOfDay(), nowDateTime, ministerUser, assistants, deptUsers);
        });

    }

    //    @Scheduled(cron = "0 40 23 * * ?")
    //    public void ScheduledSendTeamDailyReport1() {
    //        log.info("ScheduledSendTeamDailyReport1 started at {}", LocalDateTime.now());
    //        sendTeamDailyReport();
    //        log.info("ScheduledSendTeamDailyReport1 finished at {}", LocalDateTime.now());
    //    }

    @Scheduled(cron = "0 0/30 1-23 * * ?")
    public void ScheduledSendTeamDailyReport2() {
        log.info("ScheduledSendTeamDailyReport2 started at {}", LocalDateTime.now());
        sendTeamDailyReport();
        log.info("ScheduledSendTeamDailyReport2 finished at {}", LocalDateTime.now());
    }

    @Scheduled(cron = "0 15 10 * * ?")
    public void dailySummarize() {

        log.info("dailySummarize started at {}", LocalDateTime.now());
        LocalDate yesterday = LocalDate.now().minusDays(1);
        //查询部门下的所有用户
        List<DeptUsersResp> deptWithUsersAndAccounts = deptService.getDeptWithUsersAndAccounts(MINISTER_ROLE_CODE);

        deptWithUsersAndAccounts.forEach(dept -> {
            //根据用户角色对部门用户进行分组
            Map<String, List<UserWithRolesAndAccountsResp>> usersByRole = dept.getUsers()
                .stream()
                .flatMap(user -> user.getRoles()
                    .stream()
                    .map(role -> new AbstractMap.SimpleEntry<>(role.getCode(), user)))
                .collect(Collectors.groupingByConcurrent(Map.Entry::getKey, Collectors
                    .mapping(Map.Entry::getValue, Collectors.toList())));
            // 获取所有账号的username与用户的映射
            Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap = dept.getUsers()
                .stream()
                .flatMap(user -> user.getAccounts()
                    .stream()
                    .map(account -> new AbstractMap.SimpleEntry<>(account.getUsername(), user)))
                .collect(Collectors.toConcurrentMap(Map.Entry::getKey, Map.Entry::getValue));

            var ministerUser = usersByRole.get(MINISTER_ROLE_CODE).get(0);
            //建立团队之间账号的联系
            Map<String, String> accountNameWithTopAgentName = new HashMap<>();
            dept.getUsers()
                .stream()
                .flatMap(userWithRolesAndAccountsResp -> userWithRolesAndAccountsResp.getAccounts().stream())
                .forEach(accountResp -> ministerUser.getAccounts()
                    .stream()
                    .filter(ministerAccount -> Objects.equals(accountResp.getPlatformId(), ministerAccount
                        .getPlatformId()))
                    .findFirst()
                    .ifPresent(ministerAccount -> accountNameWithTopAgentName.put(accountResp
                        .getUsername(), ministerAccount.getUsername())));

            //获取账号不为空的用户
            var deptUsers = dept.getUsers().stream().filter(user -> CollUtil.isNotEmpty(user.getAccounts())).toList();
            var assistants = usersByRole.get(ASSISTANT_ROLE_CODE);

            sendDailyReport(yesterday, yesterday.atStartOfDay(), LocalDateTime
                .of(yesterday, LocalTime.MAX), ministerUser, assistants, deptUsers);
            //保存数据
            saveData(ministerUser, deptUsers, yesterday, accountNameWithTopAgentName);
            //getPayFailedMember(ministerUser, accountUsernameToUserMap, yesterday);
            sendFinance(LocalDate.now(), accountUsernameToUserMap, ministerUser);
        });
        log.info("dailySummarize finished at {}", LocalDateTime.now());
    }

    @Scheduled(cron = "1 0 0 * * ?")
    public void financeTask() {

        log.info("dailySummarize started at {}", LocalDateTime.now());
        LocalDate yesterday = LocalDate.now().minusDays(1);
        //查询部门下的所有用户
        List<DeptUsersResp> deptWithUsersAndAccounts = deptService.getDeptWithUsersAndAccounts(MINISTER_ROLE_CODE);

        deptWithUsersAndAccounts.forEach(dept -> {
            //根据用户角色对部门用户进行分组
            Map<String, List<UserWithRolesAndAccountsResp>> usersByRole = dept.getUsers()
                .stream()
                .flatMap(user -> user.getRoles()
                    .stream()
                    .map(role -> new AbstractMap.SimpleEntry<>(role.getCode(), user)))
                .collect(Collectors.groupingByConcurrent(Map.Entry::getKey, Collectors
                    .mapping(Map.Entry::getValue, Collectors.toList())));
            // 获取所有账号的username与用户的映射
            Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap = dept.getUsers()
                .stream()
                .flatMap(user -> user.getAccounts()
                    .stream()
                    .map(account -> new AbstractMap.SimpleEntry<>(account.getUsername(), user)))
                .collect(Collectors.toConcurrentMap(Map.Entry::getKey, Map.Entry::getValue));

            var ministerUser = usersByRole.get(MINISTER_ROLE_CODE).get(0);
            //建立团队之间账号的联系
            Map<String, String> accountNameWithTopAgentName = new HashMap<>();
            dept.getUsers()
                .stream()
                .flatMap(userWithRolesAndAccountsResp -> userWithRolesAndAccountsResp.getAccounts().stream())
                .forEach(accountResp -> ministerUser.getAccounts()
                    .stream()
                    .filter(ministerAccount -> Objects.equals(accountResp.getPlatformId(), ministerAccount
                        .getPlatformId()))
                    .findFirst()
                    .ifPresent(ministerAccount -> accountNameWithTopAgentName.put(accountResp
                        .getUsername(), ministerAccount.getUsername())));

            //获取账号不为空的用户
            var deptUsers = dept.getUsers().stream().filter(user -> CollUtil.isNotEmpty(user.getAccounts())).toList();
            //保存数据
            saveData(ministerUser, deptUsers, yesterday, accountNameWithTopAgentName);
            //getPayFailedMember(ministerUser, accountUsernameToUserMap, yesterday);
            sendFinance(LocalDate.now(), accountUsernameToUserMap, ministerUser);
        });
        log.info("dailySummarize finished at {}", LocalDateTime.now());
    }

    private void sendTeamDailyReport() {
        LocalDateTime nowDateTime = LocalDateTime.now();
        LocalDate nowDate = LocalDate.now();
        //查询部门下的所有用户
        List<DeptUsersResp> deptWithUsersAndAccounts = deptService.getDeptWithUsersAndAccounts(MINISTER_ROLE_CODE);

        deptWithUsersAndAccounts.forEach(dept -> {
            //根据用户角色对部门用户进行分组
            Map<String, List<UserWithRolesAndAccountsResp>> usersByRole = dept.getUsers()
                .stream()
                .flatMap(user -> user.getRoles()
                    .stream()
                    .map(role -> new AbstractMap.SimpleEntry<>(role.getCode(), user)))
                .collect(Collectors.groupingByConcurrent(Map.Entry::getKey, Collectors
                    .mapping(Map.Entry::getValue, Collectors.toList())));

            var userWithRolesAndAccountsResps = usersByRole.get(MINISTER_ROLE_CODE);
            userWithRolesAndAccountsResps.forEach(ministerUser -> generateAndSendTeamReport(ministerUser, nowDate
                .atStartOfDay(), nowDateTime, null));
        });
    }

    public void sendFinance(LocalDate date,
                            Map<String, UserWithRolesAndAccountsResp> userWithRolesAndAccountsRespMap,
                            UserWithRolesAndAccountsResp ministerUser) {
        List<FinanceDO> finances = financeService.getFinanceByDate(date);
        Map<UserWithRolesAndAccountsResp, List<FinanceDO>> userFinances = finances.stream()
            .filter(finance -> userWithRolesAndAccountsRespMap.containsKey(finance.getAgentName()))
            .collect(Collectors.groupingBy(finance -> userWithRolesAndAccountsRespMap.get(finance.getAgentName())));

        userFinances.forEach((user, userFinancesList) -> {
            if (user != null && DisEnableStatusEnum.ENABLE.equals(user.getNeedNotify()) && !user.getUserId()
                .equals(ministerUser.getUserId())) {
                String message = telegramMessageService.buildFinanceMessage(userFinancesList);
                String botToken = StrUtil.isEmpty(user.getBotToken()) ? ministerUser.getBotToken() : user.getBotToken();
                telegramMessageService.sendMessage(botToken, user.getReportIds(), message);
            }
            //telegramMessageService.sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, message);
        });
    }

    private List<CompletableFuture<MemberPagination<List<TeamMember>>>> createFuturesForPagination(AccountResp account,
                                                                                                   LocalDate date,
                                                                                                   int sumReg) {
        int pageSize = 100;
        int totalPages = (int)Math.ceil((double)sumReg / pageSize);

        return IntStream.range(0, totalPages).mapToObj(page -> {
            int currentPageSize = page == totalPages - 1
                ? (sumReg % pageSize == 0 ? pageSize : sumReg % pageSize)
                : pageSize;
            TeamMemberReq memberListReq = TeamMemberReq.builder()
                .registerStartDate(date)
                .registerEndDate(date)
                .startDate(date)
                .endDate(date)
                .registerSort(1)
                .status(1)
                .pageSize(currentPageSize)
                .pageNum(page + 1)
                .build();
            return completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_TEAM_LIST_URL, memberListReq, new ParameterizedTypeReference<ApiResponse<MemberPagination<List<TeamMember>>>>() {
                });
        }).toList();
    }

    /**
     * 查询存款失败用户,并发送消息
     *
     * @param date 日期
     */
    private void getPayFailedMember(UserWithRolesAndAccountsResp ministerUser,
                                    Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap,
                                    LocalDate date) {

        List<CompletableFuture<List<TeamMember>>> accountFutureList = new ArrayList<>();

        ministerUser.getAccounts().forEach(account -> {
            int sumReg = statsService.countNewRegNum(account.getUsername(), date);
            sumReg = (sumReg == 0) ? 100 : sumReg;

            List<CompletableFuture<MemberPagination<List<TeamMember>>>> paginationFutures = createFuturesForPagination(account, date, sumReg);
            CompletableFuture<Void> allPaginationFutures = CompletableFuture.allOf(paginationFutures
                .toArray(new CompletableFuture[0]));

            CompletableFuture<List<TeamMember>> aggregatedMembersFuture = allPaginationFutures
                .thenApply(v -> paginationFutures.stream().map(CompletableFuture::join).flatMap(memberPagination -> {
                    List<TeamMember> members = memberPagination.getList();
                    log.info("members size:{}", members.size());
                    return members.stream();
                }).collect(Collectors.toList()));

            CompletableFuture<List<TeamMember>> filteredMembersFuture = aggregatedMembersFuture
                .thenApplyAsync(members -> members.stream()
                    .filter(member -> member.getDeposit() != null && member.getDeposit()
                        .compareTo(BigDecimal.ZERO) == 0)
                    .collect(Collectors.toList()), asyncTaskExecutor);

            CompletableFuture<List<TeamMember>> membersWithFailedPayFuture = filteredMembersFuture
                .thenComposeAsync(membersWithoutDep -> {
                    List<CompletableFuture<TeamMember>> payRecordFutures = membersWithoutDep.stream()
                        .map(memberWithoutDep -> {
                            PayRecordsListReq req = PayRecordsListReq.builder()
                                .startDate(date)
                                .endDate(date)
                                .pageSize(10)
                                .memberName(memberWithoutDep.getName())
                                .build();
                            return fetchPaginationPayRecordWithRetry(account, req).thenApplyAsync(pagination -> {
                                if (CollUtil.isNotEmpty(pagination.getList()) && pagination.getList()
                                    .stream()
                                    .noneMatch(payRecord -> payRecord.getPayStatus() == 2)) {
                                    return memberWithoutDep;
                                } else {
                                    return null;
                                }
                            }, asyncTaskExecutor);
                        })
                        .toList();

                    return CompletableFuture.allOf(payRecordFutures.toArray(new CompletableFuture[0]))
                        .thenApply(v -> payRecordFutures.stream()
                            .map(CompletableFuture::join)
                            .filter(Objects::nonNull)
                            .collect(Collectors.toList()));
                }, asyncTaskExecutor);

            CompletableFuture<List<TeamMember>> notificationFuture = membersWithFailedPayFuture
                .thenApplyAsync(membersWithoutDep -> {
                    if (CollUtil.isNotEmpty(membersWithoutDep)) {
                        Map<String, List<TeamMember>> groupByTopAgentName = membersWithoutDep.stream()
                            .collect(Collectors.groupingBy(TeamMember::getTopAgentName));
                        groupByTopAgentName.forEach((accountName, accountMembers) -> {
                            String notification = telegramMessageService
                                .buildFailedPayMessage(accountName, accountMembers);
                            UserWithRolesAndAccountsResp currUser = accountUsernameToUserMap.get(accountName);
                            if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
                                String botToken = StrUtil.isEmpty(currUser.getBotToken())
                                    ? ministerUser.getBotToken()
                                    : currUser.getBotToken();
                                telegramMessageService.sendMessage(botToken, currUser.getReportIds(), notification);
                            }
                        });
                    }
                    return membersWithoutDep;
                }, asyncTaskExecutor);

            accountFutureList.add(notificationFuture);
        });

        CompletableFuture<Void> allAccountFutures = CompletableFuture.allOf(accountFutureList
            .toArray(new CompletableFuture[0]));
        allAccountFutures.thenRunAsync(() -> {
            List<TeamMember> allTeamMembers = accountFutureList.stream()
                .map(CompletableFuture::join)
                .flatMap(List::stream)
                .toList();
            log.info("All failed pay members size: {}", allTeamMembers.size());
            Map<String, List<TeamMember>> groupByTopAgentName = new TreeMap<>(allTeamMembers.stream()
                .collect(Collectors.groupingBy(TeamMember::getTopAgentName)));
            StringBuilder combinedNotification = new StringBuilder();
            groupByTopAgentName.forEach((accountName, accountMembers) -> {
                String notification = telegramMessageService.buildFailedPayMessage(accountName, accountMembers);
                combinedNotification.append(notification).append("\n");
            });

            telegramMessageService
                .sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, combinedNotification
                    .toString());

            if (DisEnableStatusEnum.ENABLE.equals(ministerUser.getNeedNotify())) {
                telegramMessageService.sendMessage(ministerUser.getBotToken(), ministerUser
                    .getReportIds(), combinedNotification.toString());
            }
        }, asyncTaskExecutor).exceptionally(ex -> {
            log.error("Error collecting and processing data", ex);
            return null;
        });
    }

    private CompletableFuture<Pagination<List<PayRecord>>> fetchPaginationPayRecordWithRetry(AccountResp account,
                                                                                             PayRecordsListReq req) {
        return CompletableFuture.supplyAsync(() -> completableFutureWebClientService
            .fetchDataForAccount(account, ApiPathConstants.PAY_RECORDS_LIST_URL, req, new ParameterizedTypeReference<ApiResponse<Pagination<List<PayRecord>>>>() {
            }), asyncTaskExecutor).thenCompose(future -> future).exceptionallyCompose(ex -> {
                log.error("Error fetching pay records, retrying...", ex);
                return fetchPaginationPayRecordWithRetry(account, req);
            });
    }

    private CompletableFuture<PayRecordList<List<PayRecord>>> fetchPayRecordsWithRetry(AccountResp account,
                                                                                       PayRecordListReq req) {
        //华体会接口限流严重,先加上
        if (PLATFORM_HTH.equals(account.getPlatformName())) {
            RateLimiter rateLimiter = rateLimiterConfig.getRateLimiter("hth", 0.2);
            log.info("限流器触发中....");
            rateLimiter.acquire();  // 通过限流器限流
        }
        return CompletableFuture.supplyAsync(() -> completableFutureWebClientService
            .fetchDataForAccount(account, ApiPathConstants.PAY_RECORD_LIST_URL, req, new ParameterizedTypeReference<ApiResponse<PayRecordList<List<PayRecord>>>>() {
            }), asyncTaskExecutor).thenCompose(future -> future).exceptionallyCompose(ex -> {
                log.error("Error fetching pay records", ex);
                return null;
            });
    }

    private void sendDailyReport(LocalDate reportDate,
                                 LocalDateTime startDateTime,
                                 LocalDateTime endDateTime,
                                 UserWithRolesAndAccountsResp ministerUser,
                                 List<UserWithRolesAndAccountsResp> assistants,
                                 List<UserWithRolesAndAccountsResp> deptUsers) {

        List<CompletableFuture<Void>> tasks = new ArrayList<>();
        tasks.add(generateAndSendTeamReport(ministerUser, startDateTime, endDateTime, assistants));

        AgentDataVisualListReq agentDataVisualListReq = AgentDataVisualListReq.builder().monthDate(reportDate).build();

        deptUsers.forEach(deptUser -> tasks
            .add(processDeptUser(deptUser, ministerUser, assistants, agentDataVisualListReq, reportDate)));

        CompletableFuture<Void> allTasks = CompletableFuture.allOf(tasks.toArray(new CompletableFuture[0]));
        allTasks.join();
    }

    /**
     * 构建表格
     *
     * @param ministerUser  部长
     * @param startDateTime 起始时间
     * @param endDateTime   结束时间
     * @return CompletableFuture<Void>
     */

    private CompletableFuture<Void> generateAndSendTeamReport(UserWithRolesAndAccountsResp ministerUser,
                                                              LocalDateTime startDateTime,
                                                              LocalDateTime endDateTime,
                                                              List<UserWithRolesAndAccountsResp> assistants) {
        log.info("Starting to generate and send team report for minister user: {}", ministerUser.getUsername());

        return CompletableFuture.runAsync(() -> {
            ConcurrentLinkedQueue<String[]> rows = new ConcurrentLinkedQueue<>();
            List<AccountResp> accounts = ministerUser.getAccounts();

            AtomicIntegerArray totals = new AtomicIntegerArray(2);
            TeamInfoReq teamInfoReq = TeamInfoReq.builder().startDate(startDateTime).endDate(endDateTime).build();

            List<CompletableFuture<Void>> futures = accounts.stream()
                .map(accountResp -> teamService.getLatestTeamInfoAsync(accountResp, teamInfoReq)
                    .thenAcceptAsync(team -> {
                        int totalNewMember = team.getList().stream().mapToInt(TeamAccount::getSubMemberNum).sum();
                        int totalNewFirstDeposit = team.getList()
                            .stream()
                            .mapToInt(TeamAccount::getFirstDepositNum)
                            .sum();

                        totals.addAndGet(0, totalNewMember);
                        totals.addAndGet(1, totalNewFirstDeposit);

                        String percent = getPercent(totalNewFirstDeposit, totalNewMember);
                        rows.add(new String[] {accountResp.getPlatformName(), String.valueOf(totalNewMember), String
                            .valueOf(totalNewFirstDeposit), percent});
                    }, asyncTaskExecutor)
                    .exceptionally(ex -> {
                        log.error("Error fetching team info for account: {}", accountResp.getPlatformName(), ex);
                        return null;
                    }))
                .toList();

            CompletableFuture<Void> allFutures = CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]));
            allFutures.join();

            log.info("Completed all async tasks for minister user: {}", ministerUser.getUsername());

            // 对 rows 列表进行排序
            List<String[]> sortedRows = new ArrayList<>(rows);
            sortedRows.sort(Comparator.comparing((String[] row) -> row[0].length()).thenComparing(row -> row[0]));

            sortedRows.add(new String[] {"总计", String.valueOf(totals.get(0)), String.valueOf(totals.get(1)),
                getPercent(totals.get(1), totals.get(0))});

            String message = TableFormatter.formatTableAsHtml(sortedRows);

            telegramMessageService.sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, message);

            if (ministerUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                telegramMessageService.sendMessage(ministerUser.getBotToken(), ministerUser.getReportIds(), message);
            }

            // 发送消息给助理
            if (!CollUtil.isEmpty(assistants)) {
                assistants.forEach(assistant -> {
                    if (assistant.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                        telegramMessageService.sendMessage(assistant.getBotToken(), assistant.getReportIds(), message);
                        telegramMessageService.sendMessage(assistant.getBotToken(), assistant.getReportIds(), String
                            .format("1组新增：%s\n在岗人数：", totals.get(1)));
                        log.info("Sent report to assistant: {}", assistant.getUsername());
                    }
                });
            }
        }, asyncTaskExecutor).exceptionally(ex -> {
            log.error("Error generating and sending team report", ex);
            return null;
        });
    }

    /**
     * 查询部门所有用户的报表
     *
     * @param deptUser               部门下所有用户对象
     * @param agentDataVisualListReq 请求参数
     * @param reportDate             日期
     * @param ministerUser           上级
     * @return CompletableFuture<Void>
     */
    private CompletableFuture<Void> processDeptUser(UserWithRolesAndAccountsResp deptUser,
                                                    UserWithRolesAndAccountsResp ministerUser,
                                                    List<UserWithRolesAndAccountsResp> assistants,
                                                    AgentDataVisualListReq agentDataVisualListReq,
                                                    LocalDate reportDate) {
        return CompletableFuture.runAsync(() -> {
            List<AccountResp> currUserAccounts = deptUser.getAccounts();

            List<CompletableFuture<Statics>> futures = currUserAccounts.stream()
                // 团队账号暂时不用具体的代理线数据
                .filter(accountResp -> !accountResp.getIsTeam())
                .map(currAccount -> agentDataVisualListService
                    .getAgentDataVisualList(currAccount, agentDataVisualListReq)
                    .thenApplyAsync(agentData -> agentData.getCurData()
                        .stream()
                        .filter(data -> data.getStaticsDate().equals(reportDate))
                        .findFirst()
                        .orElseThrow(() -> new BusinessException("No data found for report date")))
                    .exceptionally(ex -> {
                        log.error("Error fetching data for account {}: {}", currAccount.getId(), ex.getMessage());
                        return null;
                    }))
                .toList();

            CompletableFuture<Void> userStaticsFuture = CompletableFuture.allOf(futures
                .toArray(new CompletableFuture[0]));
            userStaticsFuture.thenRunAsync(() -> {
                List<Statics> agentDataList = futures.stream().map(future -> {
                    try {
                        return future.join();
                    } catch (Exception ex) {
                        log.error("Error joining future", ex);
                        return null;
                    }
                }).filter(Objects::nonNull).collect(Collectors.toList());
                // 构造消息体
                String message = telegramMessageService.buildDailyReportMessage(agentDataList);
                if (StrUtil.isNotBlank(message) && deptUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                    String botToken = StrUtil.isEmpty(deptUser.getBotToken())
                        ? ministerUser.getBotToken()
                        : deptUser.getBotToken();
                    telegramMessageService.sendMessage(botToken, deptUser.getReportIds(), message);
                }
                // 发送消息给助理
                if (!CollUtil.isEmpty(assistants)) {
                    assistants.forEach(assistant -> {
                        if (assistant.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                            if (message.contains("ky3tg107101")) {
                                var assistantMsg = message.replace("ky3tg107101", "心食谱\nky3tg107101");
                                assistantMsg = assistantMsg.replace("ky3tg107105", "信息网\nky3tg107105");
                                int index = assistantMsg.indexOf("ky3tg107108");
                                if (index != -1) {
                                    assistantMsg = assistantMsg.substring(0, index + 1);
                                }
                                telegramMessageService.sendMessage(assistant.getBotToken(), assistant
                                    .getReportIds(), assistantMsg);
                            }
                        }
                    });
                }

            }, asyncTaskExecutor).exceptionally(ex -> {
                log.error("Error collecting and processing data", ex);
                return null;
            }).join();
        }, asyncTaskExecutor).exceptionally(ex -> {
            log.error("Error processing dept user", ex);
            return null;
        });
    }

    private void saveData(UserWithRolesAndAccountsResp ministerUser,
                          List<UserWithRolesAndAccountsResp> deptUsers,
                          LocalDate reportDate,
                          Map<String, String> accountUsernameWithTopAgentName) {
        // 获取传入年月
        YearMonth inputYearMonth = YearMonth.from(reportDate);

        // 获取当前年月和上个月的年月
        YearMonth currentYearMonth = YearMonth.now();
        YearMonth previousYearMonth = currentYearMonth.minusMonths(1);

        if (inputYearMonth.equals(currentYearMonth) || inputYearMonth.equals(previousYearMonth)) {
            List<CompletableFuture<Void>> tasks = new ArrayList<>();

            TeamFinanceReq teamFinanceReq = TeamFinanceReq.builder()
                .pageNum(1)
                .pageSize(999)
                .commissionDate(reportDate)
                .build();

            // 异步处理 ministerUserAccounts
            CompletableFuture<Void> ministerAccountsFuture = CompletableFuture.runAsync(() -> {
                List<CompletableFuture<Void>> accountFutures = ministerUser.getAccounts()
                    .stream()
                    .map(accountResp -> completableFutureFinanceService.getTeamFinance(accountResp, teamFinanceReq)
                        .thenAcceptAsync(financePagination -> {
                            List<FinanceDO> financeReqList = financePagination.getList().stream().map(finance -> {
                                FinanceDO financeDO = new FinanceDO();
                                BeanUtil.copyProperties(finance, financeDO);
                                financeDO.setTopAgentName(accountResp.getUsername());
                                return financeDO;
                            }).toList();
                            financeService.addAll(financeReqList);
                            FinanceSumReq financeSumReq = new FinanceSumReq();
                            BeanUtil.copyProperties(financePagination.getTotalSumVo(), financeSumReq);
                            financeSumService.add(financeSumReq);
                        }, asyncTaskExecutor)
                        .exceptionally(ex -> {
                            log.error("Error processing minister accounts for account {}", accountResp
                                .getUsername(), ex);
                            return null;
                        }))
                    .toList();
                CompletableFuture.allOf(accountFutures.toArray(new CompletableFuture[0])).join();
            }, asyncTaskExecutor).exceptionally(ex -> {
                log.error("Error processing minister accounts", ex);
                return null;
            });
            tasks.add(ministerAccountsFuture);

            // 异步处理 deptUsers
            AgentDataVisualListReq agentDataVisualListReq = AgentDataVisualListReq.builder()
                .monthDate(reportDate)
                .build();

            deptUsers.forEach(deptUser -> {
                CompletableFuture<Void> deptUserFuture = CompletableFuture.runAsync(() -> {
                    List<AccountResp> currUserAccounts = deptUser.getAccounts();
                    List<CompletableFuture<StatsDO>> futures = currUserAccounts.stream()
                        .map(currAccount -> agentDataVisualListService
                            .getAgentDataVisualList(currAccount, agentDataVisualListReq)
                            .thenApplyAsync(agentData -> {
                                Statics statics = agentData.getCurData()
                                    .stream()
                                    .filter(data -> data.getStaticsDate().equals(reportDate))
                                    .findFirst()
                                    .orElseThrow(() -> new BusinessException("No data found for report date"));
                                StatsDO statsDO = new StatsDO();
                                BeanUtil.copyProperties(statics, statsDO);
                                statsDO.setTopAgentName(accountUsernameWithTopAgentName.get(currAccount.getUsername()));
                                return statsDO;
                            }, asyncTaskExecutor)
                            .exceptionally(ex -> {
                                log.error("Error fetching data for account {}: {}", currAccount.getId(), ex
                                    .getMessage());
                                return null;
                            }))
                        .toList();

                    List<StatsDO> list = futures.stream()
                        .map(CompletableFuture::join)
                        .filter(Objects::nonNull)
                        .collect(Collectors.toList());

                    statsService.addAll(list);
                }, asyncTaskExecutor).exceptionally(ex -> {
                    log.error("Error processing dept user accounts", ex);
                    return null;
                });
                tasks.add(deptUserFuture);
            });
            CompletableFuture<Void> allTasks = CompletableFuture.allOf(tasks.toArray(new CompletableFuture[0]));
            allTasks.join();
        } else {
            throw new BusinessException("只允许查询当月以及上个月的数据");
        }

    }

    public static String getPercent(int x, int y) {
        double d1 = x * 1.0;
        double d2 = y * 1.0;
        NumberFormat percentInstance = NumberFormat.getPercentInstance();
        percentInstance.setMinimumFractionDigits(2);
        return percentInstance.format(d1 / d2);
    }
}
