/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.service;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.constant.ApiPathConstants;
import com.zayac.admin.req.ActiveListReq;
import com.zayac.admin.req.MemberDetailsReq;
import com.zayac.admin.req.PayRecordsListReq;
import com.zayac.admin.req.team.TeamInfoReq;
import com.zayac.admin.req.team.TeamMemberListReq;
import com.zayac.admin.resp.*;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccountWithChange;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.model.resp.UserWithRolesAndAccountsResp;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Service;
import top.continew.starter.cache.redisson.util.RedisUtils;
import top.continew.starter.core.exception.BusinessException;

import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.zayac.admin.common.constant.CacheConstants.SUCCESSFULLY_PAYED_ACCOUNTNAME;
import static com.zayac.admin.utils.CommonUtils.findChangedTeamAccount;

@Slf4j
@Service
@RequiredArgsConstructor
public class DepositService {
    private final CompletableFutureWebClientService completableFutureWebClientService;
    private final TelegramMessageService telegramMessageService;
    private final TeamService teamService;

    private static final String BOT_TOKEN = "6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto";
    private static final Long TELEGRAM_CHAT_ID = 6054562838L;

    public CompletableFuture<Void> processDeposits(UserWithRolesAndAccountsResp ministerUser,
                                                   Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap,
                                                   AccountResp account,
                                                   Team currentTeam,
                                                   Team previousTeam,
                                                   LocalDate nowDate,
                                                   LocalDateTime nowDateTime,
                                                   Executor asyncTaskExecutor) {
        if (previousTeam == null || currentTeam.getFirstDepositNum() <= previousTeam.getFirstDepositNum()) {
            return CompletableFuture.completedFuture(null);
        }

        return processDepositRecords(ministerUser, accountUsernameToUserMap, account, currentTeam, previousTeam, nowDate, nowDateTime, asyncTaskExecutor);
    }

    private CompletableFuture<Void> processDepositRecords(UserWithRolesAndAccountsResp ministerUser,
                                                          Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap,
                                                          AccountResp account,
                                                          Team currentTeam,
                                                          Team previousTeam,
                                                          LocalDate nowDate,
                                                          LocalDateTime nowDateTime,
                                                          Executor asyncTaskExecutor) {
        List<TeamAccountWithChange> hasNewDepositAccounts = findChangedTeamAccount(previousTeam, currentTeam).stream()
            .filter(teamAccountWithChange -> teamAccountWithChange.getNewDepositNum() > 0)
            .toList();

        List<CompletableFuture<Void>> allTasks = hasNewDepositAccounts.stream()
            .map(accountWithChange -> processAccountChanges(accountWithChange, ministerUser, accountUsernameToUserMap, account, nowDate, nowDateTime, asyncTaskExecutor))
            .toList();

        return CompletableFuture.allOf(allTasks.toArray(new CompletableFuture[0]));
    }

    private CompletableFuture<Void> processAccountChanges(TeamAccountWithChange accountWithChange,
                                                          UserWithRolesAndAccountsResp ministerUser,
                                                          Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap,
                                                          AccountResp account,
                                                          LocalDate nowDate,
                                                          LocalDateTime nowDateTime,
                                                          Executor asyncTaskExecutor) {
        ActiveListReq req = ActiveListReq.builder()
            .pageNum(1)
            .activeType(3)
            .topAgentName(accountWithChange.getAgentName())
            .date(nowDate)
            .pageSize(9999)
            .isRest(false)
            .build();

        return completableFutureWebClientService
            .fetchDataForAccount(account, ApiPathConstants.ACTIVE_LIST, req, new ParameterizedTypeReference<ApiResponse<Pagination<List<ActiveListResp>>>>() {
            })
            .thenApply(response -> {
                if (response == null || response.getList() == null) {
                    log.warn("Fetched active list responses are null");
                    return Collections.<ActiveListResp>emptyList();
                }
                return response.getList();
            })
            .thenComposeAsync(activeListResps -> {
                if (activeListResps.isEmpty()) {
                    TeamInfoReq teamInfoReq = TeamInfoReq.builder()
                        .startDate(nowDate.atStartOfDay())
                        .endDate(nowDate.atTime(LocalTime.MAX))
                        .agentName(accountWithChange.getAgentName())
                        .pageNum(1)
                        .pageSize(1)
                        .build();
                    log.info("未能获取到{}首存信息", accountWithChange.getAgentName());
                    CompletableFuture<Void> teamInfoFuture = teamService.getLatestTeamInfoAsync(account, teamInfoReq)
                        .thenAcceptAsync(team -> accountWithChange.setNowDateDepNum(team.getList()
                            .get(0)
                            .getFirstDepositNum()), asyncTaskExecutor);

                    return teamInfoFuture.thenApply(v -> Collections.<ActiveListResp>emptyList());
                }

                // 过滤并排序
                List<ActiveListResp> sortedList = activeListResps.stream()
                    .filter(resp -> resp.getFirstPayTime() != null && resp.getFirstPayTime()
                        .isAfter(nowDateTime.minusMinutes(5)))
                    .sorted(Comparator.comparing(ActiveListResp::getFirstPayTime))
                    .collect(Collectors.toList());

                long nowDateNewDep = activeListResps.stream()
                    .filter(mem -> mem.getFirstPayTime().isAfter(nowDate.atStartOfDay()))
                    .count();
                accountWithChange.setNowDateDepNum((int)nowDateNewDep);

                // 截取后N个元素
                List<ActiveListResp> activeListRespList = ListUtil.sub(sortedList, -accountWithChange
                    .getNewDepositNum(), sortedList.size());

                // 异步处理每个响应
                List<CompletableFuture<Void>> futures = activeListRespList.stream().map(resp -> {
                    if (resp.getDeposit().compareTo(BigDecimal.ZERO) == 0) {
                        LocalDate startDate = nowDateTime.minusHours(1).toLocalDate();
                        PayRecordsListReq payRecordsListReq = PayRecordsListReq.builder()
                            .startDate(startDate)
                            .endDate(nowDate)
                            .memberName(resp.getName())
                            .pageSize(10)
                            .payState(2)
                            .agentName(accountWithChange.getAgentName())
                            .build();

                        return completableFutureWebClientService
                            .fetchDataForAccount(account, ApiPathConstants.PAY_RECORDS_LIST_URL, payRecordsListReq, new ParameterizedTypeReference<ApiResponse<Pagination<List<PayRecord>>>>() {
                            })
                            .thenApply(response -> {
                                if (response == null || response.getList() == null) {
                                    log.warn("No pay records found for member: {}", resp.getName());
                                    return Collections.<PayRecord>emptyList();
                                }
                                return response.getList();
                            })
                            .thenAcceptAsync(payRecords -> {
                                if (!payRecords.isEmpty()) {
                                    resp.setDeposit(payRecords.get(0).getScoreAmount());
                                }
                            }, asyncTaskExecutor);
                    }
                    return CompletableFuture.<Void>completedFuture(null);
                }).toList();

                return CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]))
                    .thenApply(v -> activeListRespList);
            }, asyncTaskExecutor)
            .thenAcceptAsync(activeListRespList -> {
                String depositResults = buildDepositResults(activeListRespList);
                String notification = buildNotificationMessage(accountWithChange, depositResults);
                sendNotifications(accountWithChange, ministerUser, accountUsernameToUserMap, notification);
            }, asyncTaskExecutor)
            .exceptionally(ex -> {
                log.error("Error processing account changes for account: {}", account, ex);
                return null;
            });
    }

    private String buildDepositResults(List<ActiveListResp> activeListRespList) {
        StringBuilder depositResults = new StringBuilder();
        activeListRespList.forEach(activeListResp -> depositResults.append(telegramMessageService
            .buildDepositResultsMessage(activeListResp.getName(), activeListResp.getDeposit())));
        return depositResults.toString();
    }

    private String buildNotificationMessage(TeamAccountWithChange accountWithChange, String depositResults) {
        return telegramMessageService.buildDepositMessage(accountWithChange, depositResults);
    }

    private void sendNotifications(TeamAccountWithChange accountWithChange,
                                   UserWithRolesAndAccountsResp ministerUser,
                                   Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap,
                                   String notification) {
        var currUser = accountUsernameToUserMap.get(accountWithChange.getAgentName());
        if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify()) && !"guanzi".equals(currUser
            .getUsername())) {
            String botToken = StrUtil.isEmpty(currUser.getBotToken())
                ? ministerUser.getBotToken()
                : currUser.getBotToken();
            telegramMessageService.sendMessage(botToken, currUser.getRegAndDepIds(), notification);
        }
        telegramMessageService.sendMessage(BOT_TOKEN, TELEGRAM_CHAT_ID, notification);
    }

    //    // 凌晨的时候查询存款记录时往前减一天 防止凌晨的时候出现查询不到存款记录的问题
    //    LocalDate startDate = nowDateTime.minusHours(1).toLocalDate();
    //    PayRecordsListReq req = PayRecordsListReq.builder()
    //            .startDate(startDate)
    //            .endDate(nowDate)
    //            .pageSize(100)
    //            .payState(2)
    //            .agentName(accountWithChange.getAgentName())
    //            .build();
    //
    //    StringBuilder depositResults = new StringBuilder();
    //    AtomicInteger depositCounter = new AtomicInteger(0);
    //
    //        return completableFutureWebClientService
    //                .fetchDataForAccount(account,ApiPathConstants.PAY_RECORDS_LIST_URL,req,new ParameterizedTypeReference<ApiResponse<Pagination<List<PayRecord>>>>(){}).thenApply(Pagination::getList)
    //    .thenComposeAsync(payRecords-> processPayRecords(payRecords, accountWithChange, account, nowDate, nowDateTime, depositResults, depositCounter, asyncTaskExecutor),asyncTaskExecutor)
    //    .thenRunAsync(()->sendNotification(accountWithChange, ministerUser, accountUsernameToUserMap, depositResults),asyncTaskExecutor)
    //        .exceptionally(ex->{
    //        log.error("Error processing account changes for agent {}: {}", accountWithChange.getAgentName(), ex
    //                .getMessage());
    //        return null;
    //    });
    //}

    private CompletableFuture<Void> processPayRecords(List<PayRecord> payRecords,
                                                      TeamAccountWithChange accountWithChange,
                                                      AccountResp account,
                                                      LocalDate nowDate,
                                                      LocalDateTime nowDateTime,
                                                      StringBuilder depositResults,
                                                      AtomicInteger depositCounter,
                                                      Executor asyncTaskExecutor) {

        //根据用户名去重,保留时间最早的支付记录
        List<PayRecord> sortedPayRecords = payRecords.stream()
            .filter(record -> record.getCreatedAt().isAfter(nowDateTime.minusHours(1)))
            .collect(Collectors.collectingAndThen(Collectors.toMap(PayRecord::getName, record -> record, (
                                                                                                          existingRecord,
                                                                                                          newRecord) -> existingRecord
                                                                                                              .getCreatedAt()
                                                                                                              .isBefore(newRecord
                                                                                                                  .getCreatedAt())
                                                                                                                      ? existingRecord
                                                                                                                      : newRecord, LinkedHashMap::new), map -> map
                                                                                                                          .values()
                                                                                                                          .stream()
                                                                                                                          .sorted(Comparator
                                                                                                                              .comparing(PayRecord::getCreatedAt)
                                                                                                                              .reversed())
                                                                                                                          .filter(payRecord -> !RedisUtils
                                                                                                                              .hasKey(SUCCESSFULLY_PAYED_ACCOUNTNAME + payRecord
                                                                                                                                  .getBillNo()))
                                                                                                                          .collect(Collectors
                                                                                                                              .toList())));

        List<CompletableFuture<Void>> fetchMemberFutures = sortedPayRecords.stream()
            .map(payRecord -> fetchMemberDetails(account, payRecord.getName(), nowDate, asyncTaskExecutor)
                .thenAcceptAsync(member -> processMemberDetails(member, payRecord, accountWithChange, depositResults, depositCounter), asyncTaskExecutor)
                .exceptionally(ex -> {
                    log.error("Error fetching details for member {}: {}", payRecord.getName(), ex.getMessage());
                    return null;
                }))
            .toList();

        return CompletableFuture.allOf(fetchMemberFutures.toArray(new CompletableFuture[0]));
    }

    private void processMemberDetails(Member member,
                                      PayRecord payRecord,
                                      TeamAccountWithChange accountWithChange,
                                      StringBuilder depositResults,
                                      AtomicInteger depositCounter) {
        synchronized (this) {
            //如果从缓存中没有key,那就是新存款用户
            if (!RedisUtils.hasKey(SUCCESSFULLY_PAYED_ACCOUNTNAME + payRecord.getBillNo())) {
                //如果订单记录有存款成功但是会员的首存时间还为空,数据未同步,也是首存
                if ((member.getFirstPayAt() == null || payRecord.getCreatedAt()
                    .equals(member.getFirstPayAt())) && depositCounter.getAndIncrement() < accountWithChange
                        .getNewDepositNum()) {
                    //把存款成功的用户保存起来,设置一个小时的缓存时间 防止重复用户
                    RedisUtils.set(SUCCESSFULLY_PAYED_ACCOUNTNAME + payRecord.getBillNo(), member.getName(), Duration
                        .ofHours(1));
                    depositResults.append(telegramMessageService.buildDepositResultsMessage(member.getName(), payRecord
                        .getScoreAmount()));
                }
            }
        }
    }

    private void sendNotification(TeamAccountWithChange accountWithChange,
                                  UserWithRolesAndAccountsResp ministerUser,
                                  Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap,
                                  StringBuilder depositResults) {
        String notification = telegramMessageService.buildDepositMessage(accountWithChange, depositResults.toString());
        var currUser = accountUsernameToUserMap.get(accountWithChange.getAgentName());
        if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
            String botToken = StrUtil.isEmpty(currUser.getBotToken())
                ? ministerUser.getBotToken()
                : currUser.getBotToken();
            telegramMessageService.sendMessage(botToken, currUser.getRegAndDepIds(), notification);
        }
        telegramMessageService.sendMessage(BOT_TOKEN, TELEGRAM_CHAT_ID, notification);
    }

    private CompletableFuture<Member> fetchMemberDetails(AccountResp account,
                                                         String name,
                                                         LocalDate nowDate,
                                                         Executor asyncTaskExecutor) {
        TeamMemberListReq memberListReq = TeamMemberListReq.builder()
            .name(name)
            .startDate(nowDate)
            .endDate(nowDate)
            .status(1)
            .build();

        CompletableFuture<MemberPagination<List<Member>>> memberFuture = completableFutureWebClientService
            .fetchDataForAccount(account, ApiPathConstants.MEMBER_TEAM_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
            });

        return memberFuture.thenApplyAsync(MemberPagination::getList, asyncTaskExecutor)
            .thenApplyAsync(list -> list.stream()
                .findFirst()
                .orElseThrow(() -> new BusinessException("没有找到匹配的成员信息")), asyncTaskExecutor)
            .thenComposeAsync(member -> fetchDetailedMemberInfo(account, member.getId(), nowDate), asyncTaskExecutor);
    }

    private CompletableFuture<Member> fetchDetailedMemberInfo(AccountResp account, Long memberId, LocalDate nowDate) {
        MemberDetailsReq detailsReq = MemberDetailsReq.builder()
            .id(memberId)
            .startDate(nowDate)
            .endDate(nowDate)
            .build();

        return completableFutureWebClientService
            .fetchDataForAccount(account, ApiPathConstants.MEMBER_DETAIL_URL, detailsReq, new ParameterizedTypeReference<>() {
            });
    }
}
