/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.service;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.text.CharPool;
import cn.hutool.core.util.StrUtil;
import com.zayac.admin.agent.model.entity.FinanceDO;
import com.zayac.admin.resp.Statics;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.resp.team.TeamAccountWithChange;
import com.zayac.admin.resp.team.TeamMember;
import com.zayac.admin.system.model.resp.UserWithRolesAndAccountsResp;
import lombok.RequiredArgsConstructor;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;
import top.continew.starter.core.exception.BusinessException;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class TelegramMessageService {
    private final RabbitTemplate rabbitTemplate;

    public void sendMessage(String botToken, Long targetId, String message) {
        if (targetId == null) {
            throw new BusinessException("targetId不能为空");
        }
        if (StrUtil.isEmpty(message)) {
            throw new BusinessException("消息不能为空");
        }
        if (!containsHTMLCharacters(message)) {
            message = escapeMarkdown(message);
        }
        String fullMessage = String.format("%s|%s|%s", botToken, targetId, message);
        this.rabbitTemplate.convertAndSend("message_queue", fullMessage);

    }

    public void sendMessage(String botToken, List<Long> targetIds, String message) {
        targetIds.parallelStream().forEach(targetId -> this.sendMessage(botToken, targetId, message));
    }

    public void sendMessage(String botToken, String targetIds, String message) {
        convertStringToList(targetIds).parallelStream()
            .forEach(targetId -> this.sendMessage(botToken, targetId, message));
    }

    public static List<Long> convertStringToList(String str) {
        // 去掉字符串的方括号和空格，然后按逗号分隔并转换为 List<Long>
        String cleanedStr = StrUtil.removeAll(str, CharPool.BRACKET_START, CharPool.BRACKET_END).trim();
        return Convert.toList(Long.class, StrUtil.split(cleanedStr, CharPool.COMMA));
    }

    private static String escapeMarkdown(String text) {

        List<Character> escapeChars = Arrays
            .asList('_', '[', ']', '(', ')', '~', '>', '#', '+', '-', '=', '|', '{', '}', '.', '!');
        for (Character charItem : escapeChars) {
            text = text.replace(charItem.toString(), "\\" + charItem);
        }
        return text;
    }

    private static boolean containsHTMLCharacters(String text) {
        List<String> htmlTags = Arrays.asList("<b>", "<i>", "<a>", "<code>", "<pre>");
        return htmlTags.stream().anyMatch(text::contains);
    }

    public String buildRegistrationMessage(UserWithRolesAndAccountsResp currUser,
                                           List<TeamMember> accountMembers,
                                           TeamAccount teamAccount) {

        String memberNames = accountMembers.stream().map(TeamMember::getName).collect(Collectors.joining(", "));
        if (currUser != null) {
            return String.format("👏 [%s] %s 注册: %d 会员: `%s` 本月:*%d*", currUser.getNickname(), teamAccount
                .getAgentName(), accountMembers.size(), memberNames, teamAccount.getSubMemberNum());
        }
        return String.format("👏 %s 注册: %d 会员: `%s` 本月:*%d*", teamAccount.getAgentName(), accountMembers
            .size(), memberNames, teamAccount.getSubMemberNum());
    }

    public String buildRegistrationMessage(List<TeamMember> accountMembers, TeamAccountWithChange teamAccount) {

        String memberNames = accountMembers.stream()
            .map(member -> "`" + member.getName() + "`")
            .collect(Collectors.joining(", "));
        return String.format("👏 %s 注册: %d 今日: *%d* 本月:*%d*  会员: %s ", teamAccount.getAgentName(), accountMembers
            .size(), teamAccount.getNowDateRegNum(), teamAccount.getSubMemberNum(), memberNames);
    }

    public String buildDepositMessage(TeamAccountWithChange accountWithChange, String memberInfo) {

        return String.format("🎉 %s 首存: *%d* 今日: *%d* 本月: *%d* \n%s", accountWithChange
            .getAgentName(), accountWithChange.getNewDepositNum(), accountWithChange
                .getNowDateDepNum(), accountWithChange.getFirstDepositNum(), memberInfo);
    }

    public String buildDepositResultsMessage(String name, BigDecimal scoreAmount) {
        return String.format("会员: `%s`, 金额: *%s*\n", name, scoreAmount);
    }

    public String buildFailedPayMessage(String accountName, List<TeamMember> accountMembers) {
        String memberNames = accountMembers.stream().map(TeamMember::getName).collect(Collectors.joining("\n"));
        return String.format("%s\n%s\n", accountName, memberNames);
    }

    public String buildDailyReportMessage(List<Statics> statics) {
        StringBuilder message = new StringBuilder();
        statics.forEach(stat -> {
            String formattedStat = String.format("%s\n注册: %s\n首存: %d\n日活: %d\n\n", stat.getAgentName(), stat
                .getIsNew(), stat.getFirstCount(), stat.getCountBets());
            message.append(formattedStat);
        });
        return message.toString();
    }

    public String buildFinanceMessage(List<FinanceDO> userFinancesList) {
        StringBuilder message = new StringBuilder();
        userFinancesList.forEach(financeDO -> {
            String formattedFinance = String.format("%s: *%s*\n", financeDO.getAgentName(), financeDO.getNetProfit()
                .toPlainString());
            message.append(formattedFinance);
        });
        return message.toString();
    }
}
